/**
 * @author Tobias Schulz-Hess (sourceforge@schulz-hess.de)
 *  12/04/2003 20:33:31
 */
package vqwiki.servlets;
 
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
 
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
 
import org.apache.log4j.Logger;
 
 
/*
Very Quick Wiki - WikiWikiWeb clone
Copyright (C) 2001-2002 Gareth Cronin
 
This program is free software; you can redistribute it and/or modify
it under the terms of the latest version of the GNU Lesser General
Public License as published by the Free Software Foundation;
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.
 
You should have received a copy of the GNU Lesser General Public License
along with this program (gpl.txt); if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
 
/**
 * This servlet exports all pages as static HTML-pages.
 * You can add a parameter "virutal-wiki", which then generates
 * the HTML pages on a particular virtual wiki.<p>
 *
 * @author Tobias Schulz-Hess (sourceforge@schulz-hess.de)
 */
public class ExportHTMLServlet extends LongLastingOperationServlet {
 
      /** Logging */
      private static final Logger logger = Logger.getLogger(ExportHTMLServlet.class);
 
      private Exception exception;
      
      protected LongLastingOperationInterface createThread () {
    	  
    	  ExportHTMLServletThread runnable = new ExportHTMLServletThread();
    	  
    	  runnable.setTempdir (tempdir);
    	  runnable.setImageDir(imageDir);
    	  runnable.setVirtualWiki(virtualWiki);
    	  runnable.setLocale(locale);
    	  runnable.setServletContext(servletContext);
    	  runnable.setStartingTime(startingTime);
   	  
    	  return runnable;
      }

      
      /**
       * Handle post request.
       * Generate a node file and send it back as text.
       *
       * @param request  The current http request
       * @param response What the servlet will send back as response
       *
       * @throws ServletException If something goes wrong during servlet execution
       * @throws IOException If the output stream cannot be accessed
       *
       */
      protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
       
        ServletContext ctx = getServletContext();
        try {
            File tmpDir = (File) ctx.getAttribute("javax.servlet.context.tempdir");
            tempdir = tmpDir.getPath();
        } catch (Throwable t) {
            logger.warn("'javax.servlet.context.tempdir' attribute undefined or invalid, using java.io.tmpdir", t);
            tempdir = System.getProperty("java.io.tmpdir");
        }
        imageDir = getServletContext().getRealPath("/images");
        if (request.getQueryString().indexOf("StartExport2HTML") >= 0) {
        	request.setAttribute("short", "1");
        }
        if (request.getQueryString().indexOf("UploadExport2HTML") >= 0) {
        	request.setAttribute("upload", "1");
        }
        if (request.getQueryString().indexOf("WaitExport2HTML") >= 0) {
        	request.setAttribute("norefresh", "1");
        }
        if (request.getQueryString().indexOf("WaitUploadExport2HTML") >= 0) {
        	request.setAttribute("norefresh", "1");
        	request.setAttribute("upload", "1");
        }
        
        
        super.doPost(request, response);
      }
 
      /**
       * We are done. Go to result page.
       * @see vqwiki.servlets.LongLastingOperationServlet#dispatchDone(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
       */
      protected void dispatchDone(HttpServletRequest request, HttpServletResponse response) {
            if (exception != null) {
                  error(request, response, new ServletException(exception.getMessage(), exception));
                  return;
            }
 
            try {
            	  SimpleDateFormat sdf = new SimpleDateFormat ("yyyyMMdd-HHmmss");
            	  Date now = new Date ();
            	  String dateString = sdf.format(now);
            	  
                  response.setContentType("application/zip");
                  response.setHeader("Expires", "0");
                  response.setHeader("Pragma", "no-cache");
                  response.setHeader("Keep-Alive", "timeout=15, max=100");
                  response.setHeader("Connection", "Keep-Alive");
 
                  response.setHeader("Content-Disposition", "attachment" + ";filename=" + virtualWiki + "." + dateString + ".zip;");

                  String lastExport = tempdir + File.separator + virtualWiki + ".HTML-Export.zip";
                  File lastExportFile = new File (lastExport);
                  FileInputStream in = new FileInputStream(lastExportFile);
 
                  response.setContentLength((int) lastExportFile.length());
                  OutputStream out = response.getOutputStream();
 
                  int bytesRead = 0;
                  byte byteArray[] = new byte[4096];
                  while (in.available() > 0) {
                        bytesRead = in.read(byteArray, 0, Math.min(4096, in.available()));
                        out.write(byteArray, 0, bytesRead);
                  }
                  out.flush();
                  out.close();
                                    
            } catch (Exception e) {
                  logger.fatal("Exception", e);
                  error(request, response, new ServletException(e.getMessage(), e));
            }
      }
} 