/*
Very Quick Wiki - WikiWikiWeb clone
Copyright (C) 2001-2002 Gareth Cronin

This program is free software; you can redistribute it and/or modify
it under the terms of the latest version of the GNU Lesser General
Public License as published by the Free Software Foundation;

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program (gpl.txt); if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package vqwiki.servlets;

import java.io.IOException;
import java.util.Date;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import vqwiki.utils.LongLastingOperationsManager;
import vqwiki.utils.Utilities;

/**
 * Abstract class to support long lasting operations
 *
 * This class was created on 00:24:36 23.07.2003
 *
 * @author $Author: garethc $
 */
public abstract class LongLastingOperationServlet extends VQWikiServlet {
  /** time, when everything starts */

  /** Maximum time in seconds to wait until the page is refreshed */
  protected int MAX_TIME_TO_REFRESH = 3;

  /** progress done */
  public static final int PROGRESS_DONE = 100;
  
  private static int CONTINUE = 1;
  private static int STOPNOW = 0;

  private static int UPLOAD = 1;
  private static int DISCARD = 0;
  
  private static int NO = 0;
  private static int YES = 1;
  
  /** the url, which is used to call this servlet */
  protected Date startingTime = null;
  protected String url;
  protected String virtualWiki;
  protected Locale locale = null;
  protected String imageDir = null;
  protected String tempdir = null;
  protected ServletContext servletContext = null;

  /**
   * Handle post request.
   * Generate a long lasting operation and execute it.
   *
   * @param request  The current http request
   * @param response What the servlet will send back as response
   *
   * @throws ServletException If something goes wrong during servlet execution
   * @throws IOException If the output stream cannot be accessed
   *
   */
  protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

	  ResourceBundle messages = Utilities.getMessages(locale);
	  
      String status = "In Progress...";
      try { status = messages.getString("longlasting.operationInProgress"); } catch (Exception e) {}

      String longtitle = "Please Wait...";
      try { longtitle = messages.getString("longlasting.title"); } catch (Exception e) {}

      String shorttitle = "Exportprocess Started...";
      try { shorttitle = messages.getString("shortlasting.title"); } catch (Exception e) {}
      
      String finished = "Finished!";
      try {finished = messages.getString("longlasting.operationFinished"); } catch (Exception e) {}
      
	  virtualWiki = (String) request.getAttribute("virtualWiki");
	  startingTime = new Date();
	  locale = request.getLocale();
	  servletContext = getServletContext();
	  url = request.getRequestURL().toString() + "?" + request.getQueryString();
	  if (url.indexOf("&") >= 0) {
		  url = url.substring(0, url.indexOf("&"));
	  }
        
      LongLastingOperationsManager mgr = LongLastingOperationsManager.getInstance();
      LongLastingOperationInterface thread = mgr.getThreadForId(virtualWiki);
      if ((thread != null) && (thread.getStopTime() != null)){
    	  long duration = startingTime.getTime() - thread.getStopTime().getTime();
    	  if (duration >= MAX_TIME_TO_REFRESH * 10000) {
         		mgr.removeThreadById(virtualWiki);
         		thread  = null;
    	  }
      }
      if (thread == null) {
    	thread = createThread();
    	Thread t = new Thread (thread);
        mgr.registerNewThread(virtualWiki, thread);
        t.setPriority(Thread.MIN_PRIORITY);
        t.start();
      }

      int cont = CONTINUE;
      try {
    	  cont = Integer.parseInt((String) request.getParameter("cont"));
      } catch (Exception e) {}
     
      int upload = DISCARD;
      try {
    	  upload = Integer.parseInt((String) request.getParameter("upload"));
      } catch (Exception e) {}
      try {
    	  upload = Integer.parseInt((String) request.getAttribute("upload"));
      } catch (Exception e) {}
      
      int norefresh = NO;
      try {
    	  norefresh = Integer.parseInt((String) request.getParameter("norefresh"));
      } catch (Exception e) {}
      try {
    	  norefresh = Integer.parseInt((String) request.getAttribute("norefresh"));
      } catch (Exception e) {}
      
      int shortstart =  NO;
      try {
    	  shortstart = Integer.parseInt((String) request.getParameter("short"));
      } catch (Exception e) {}
      try {
    	  shortstart = Integer.parseInt((String) request.getAttribute("short"));
      } catch (Exception e) {}
     
      
      
      // get information on the thread
      request.setAttribute("nextRefresh", new Integer(getNextRefresh(thread)));
      request.setAttribute("url", url);
      request.setAttribute("cont", new Integer(CONTINUE));
      request.setAttribute("status", status);
      request.setAttribute("title", longtitle);
      request.setAttribute("upload", new Integer(upload));

      if ((shortstart == YES)) {
          request.setAttribute("status", shorttitle);
          request.setAttribute("title", shorttitle);
    	  dispatch("/jsp/shortlastingoperation.jsp", request, response);
    	  return;
      }
      
      if (norefresh == YES) {
    	  while (thread.getProgress() < PROGRESS_DONE) {
    		  try {
    			  Object o = new Object ();
    			  synchronized (o) {
    				  o.wait(getNextRefresh(thread)*1000);
    			  }
    		  } catch (Exception e) {
    			  e.printStackTrace();
    		  }
    	  }
      }

      request.setAttribute("progress", new Integer(thread.getProgress()));
      
      if (thread.getProgress() >= PROGRESS_DONE) {
      	if ((cont == STOPNOW) && (upload == UPLOAD)) {
       		dispatchDone(request, response);
       		return;
       	} else {
            request.setAttribute("status", finished);
            request.setAttribute("title",finished);
            request.setAttribute("cont", new Integer(STOPNOW));
            request.setAttribute("nextRefresh", new Integer(0));
            if (upload == DISCARD) {
          	  	dispatch("/jsp/longlastingoperationend.jsp", request, response);
          	  	return;
            }
       	}
      }
   	  dispatch("/jsp/longlastingoperation.jsp", request, response);
  }

  /**
   * Handle get request.
   * The request is handled the same way as the post request.
   *
   * @see doPost()
   *
   * @param httpServletRequest  The current http request
   * @param httpServletResponse What the servlet will send back as response
   *
   * @throws ServletException If something goes wrong during servlet execution
   * @throws IOException If the output stream cannot be accessed
   *
   */
  protected void doGet(
      HttpServletRequest httpServletRequest,
      HttpServletResponse httpServletResponse)
      throws ServletException, IOException {
    this.doPost(httpServletRequest, httpServletResponse);
  }

  /**
   * How many seconds shall we wait until the next refresh?
   * @return int giving the number of seconds to wait for the next refresh
   */
  protected int getNextRefresh(LongLastingOperationInterface llo) {
	  /*
      double alreadyWorking = (new Date().getTime() - llo.getStartingTime().getTime())/1000.0;
      int timeToFinish;
      if (llo.getProgress() == 0) {
    	  timeToFinish = 999999;
      } else {
    	  timeToFinish = (int) ((double) alreadyWorking * ((100.0 -(double) llo.getProgress())/(double) llo.getProgress()));
      }
	  System.out.println ("alreadyWorking: " + alreadyWorking);
	  System.out.println ("timeToFinish: " + timeToFinish);
	  System.out.println ("progress: " + llo.getProgress());
	  */
	  return MAX_TIME_TO_REFRESH;
  }

  /**
   * Go to the done page
   * @param request The current servlet request
   * @param response The current servlet response
   */
  protected abstract void dispatchDone(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse);

  /**
   * Get the url, how this servlet is called
   * @return
   */
  protected String getUrl() {
    return url;
  }

  protected abstract LongLastingOperationInterface createThread ();
}

 
/*
 * Log:
 *
 * $Log: LongLastingOperationServlet.java,v $
 * Revision 1.4  2003/10/05 05:07:32  garethc
 * fixes and admin file encoding option + merge with contributions
 *
 * Revision 1.3  2003/08/20 20:41:41  mrgadget4711
 * ADD: Override refresh with norefresh=true
 *
 * Revision 1.2  2003/07/23 09:50:50  mrgadget4711
 * Fixes
 *
 * Revision 1.1  2003/07/23 00:34:26  mrgadget4711
 * ADD: Long lasting operations
 *
 * ------------END------------
 */